<?php
/**
 * The main controller class of the application shows archieves and individual
 * photos along with adding and deleting them. add, edit, delete functions are
 * protected using rdSimpleAuth
 *
 * @author Dr. Tarique Sani,  tarique@sanisoft.com
 * @copyright Copyright (c); 2006, Dr. Tarique Sani, Nagpur, India
 * @package cheeseCake
 * @version $Id: photos_controller.php 166 2006-08-01 11:08:28Z amit $
 */
class PhotosController extends AppController
{
    var $name = 'Photos';
    var $helpers = array('Html','Javascript','Cheese','Pagination');
    var $components = array('Pagination','rss');
    var $Sanitize = '';

    var $rdAccess = array('add' => array('Admin'),
                          'edit' => array('Admin'),
                          'delete'=> array('Admin')
                         );


    function index($id=null)
    {
        $id=(int)$id;
        if(empty($id)){
            $photoData = $this->Photo->findAll("ORDER BY id DESC","","",1);
            $id = !empty($photoData)?$photoData[0]['Photo']['id']:"";
        }else{
            $photoData = $this->Photo->findAll("id=$id");
        }
        /**
         * If photo was found
         */
        if(!empty($photoData)){
            $photoData[0]['Photo']['commentCount'] = count($photoData[0]['Comment']) ;
            $photoData[0]['Photo']['exif'] = unserialize($photoData[0]['Photo']['exif']);
            $filmstrip = $this->Photo->filmstrip($id,5);
            $this->set('photo',$photoData);
            $this->pageTitle = $this->siteName.' - '.$photoData[0]['Photo']['title'];
            $this->set('neighbours', $this->Photo->findNeighbours(null,'id',$id));
            unset($photoData);
        }else{
            $this->pageTitle = $this->siteName.' - Photo not found!';
            $this->set('message',"Photo you have requested was not found");
            $this->render(null,null,'views/errors/cc_die.thtml');
            exit;
        }
    }

    /**
     * Browse photos either all of them or by month, if month is empty then show
     * all of them
     * @param string $month expects format YYYY-MM
     */
    function browse($month=null)
    {
        /**
         * Sanitize month
         */
        preg_match('/([0-9]{4})-([0-9]{2})/', $month, $matches);
        $month = isset($matches[0])?$matches[0]:"";

        if(empty($month)){
            $criteria = null;
            $this->set('browsing','Last Uploaded');
        }else{
            $criteria = "DATE_FORMAT(created, '%Y-%m')='$month'";
            /**
             * $month is set show the calendar and stuff
             */
            $this->Photo->unbindModel(array('hasMany'=>array('Comment'),'hasAndBelongsToMany'=>array('Tag')));
            /**
             * Probably would be much better if I ran a straight query, but
             * findAll does allow some funky stuff
             */
            $calendar = $this->Photo->findAll('1=1 GROUP  BY DATE_FORMAT( created,  \'%Y-%m\'  )',
                                                array('DATE_FORMAT( created,  \'%Y-%m\'  )  AS month',
                                                'count( id ) as photoCount'), "created DESC");
            $this->set('calendar',$calendar);
            $this->set('browsing',"Month $month");
        }

        $this->Photo->unbindModel(array('hasMany'=>array('Comment'),'hasAndBelongsToMany'=>array('Tag')));
        list($order,$limit,$page) = $this->Pagination->init($criteria);

        $this->Photo->unbindModel(array('hasMany'=>array('Comment'),'hasAndBelongsToMany'=>array('Tag')));
        $photos = $this->Photo->findAll($criteria, array('id','filename','title'), "created DESC", $limit, $page);
        if(!empty($photos)){
            $this->set('photos',$photos);
            $this->pageTitle = $this->siteName." - Browse photos";
        }else{
            $this->pageTitle = $this->siteName.' - No Photos!';
            $this->set('message',"No photos found");
            $this->render(null,null,'views/errors/cc_die.thtml');
            exit;
        }
    }

    function tag($tag=null)
    {
        $this->Sanitize = new Sanitize;
        $tag = $this->Sanitize->sql($tag);
        $paging['total']= $this->Photo->findCountByTag($tag);
        $paging['url'] = '/photos/tag/'.$tag."/";

        list($order,$limit,$page) = $this->Pagination->init(null,$paging);

        $photos = $this->Photo->findAllByTag($tag,$limit,$page);

        if(!empty($photos)){
            $this->set('photos',$photos);
            $this->set('browsing',"Tag - $tag");
            $this->pageTitle = $this->siteName." - Browse photos for $tag";
            $this->render(null,null,'views/photos/browse.thtml');
        }else{
            $this->pageTitle = $this->siteName.' - No Photos!';
            $this->set('message',"No photos found for tag $tag");
            $this->render(null,null,'views/errors/cc_die.thtml');
            exit;
        }
    }

    function add()
    {
        vendor('exif');
        $this->pageTitle = $this->siteName." - Add a photo";
        $this->set('tags', $this->Photo->Tag->generateList(null, 'tag ASC', null, '{n}.Tag.id', '{n}.Tag.tag'));
        if (!empty($this->params['data']['Photo'])){
            /**
             * File has been uploaded and withour errors
             */
            if($this->params['form']['photoFile']['error'] > 0){
                $this->Photo->invalidate('filename');
            }else{
                /**
                 * Hack to ensure unique filenames
                 * TODO: Code to warn user about duplicate files
                 */
                $this->params['form']['photoFile']['name'] = time()."_".$this->params['form']['photoFile']['name'];
                $thumbSize = $this->Session->read('cheeseCake.thumbSize');
                $thumbType = $this->Session->read('cheeseCake.thumbType');
                if($this->Photo->savePhoto($this->params['form']['photoFile'],$thumbSize,$thumbType)){
                    $this->params['data']['Photo']['filename'] = $this->params['form']['photoFile']['name'];
                    /**
                     * Cleaning  EXIF array before seralizing does not really
                     * help solve the binary output problem
                     */
                    $exif = read_exif_data_raw(WWW_ROOT."img/photos/".$this->params['form']['photoFile']['name'],"0");
                    $this->Sanitize = new Sanitize;
                    $this->Sanitize->cleanArray($exif);
                    $exif = serialize($exif);
                    $this->params['data']['Photo']['exif'] = addslashes($exif);
                }else{
                    $this->params['data']['Photo']['filename'] = null;
                    // TODO: Write code to graciously exit if Photo::savePhoto fails for now just die()
                    die("I am sorry to fail you my master but I could not save your photo");
                }
            }

            if (!empty($this->params['data']['Photo']['Tags'])){
                $this->params['data']['Tag']['Tag'] = $this->Photo->Tag->parseTags($this->params['data']['Photo']['Tags']);
            }

            if($this->Photo->save($this->params['data'])){
                $this->Session->setFlash('Your Photo was saved successfully');
                $this->redirect("/");
            }else{
                $this->Photo->delPhoto($this->params['form']['photoFile']);
                // TODO: Write code to graciously exit if Photo::save fails for now just delete the file
            }

        }
    }

    function edit($id=null)
    {
        vendor('exif');
        $this->pageTitle = $this->siteName.' - Edit Photo';
        $this->Photo->id= $id;
        $photoData = $this->Photo->read();
        $this->set('filename',$photoData['Photo']['filename']);
        $this->set('id',$photoData['Photo']['id']);
        $this->set('tags', $this->Photo->Tag->generateList(null, 'tag ASC', null, '{n}.Tag.id', '{n}.Tag.tag'));

        // The form was not posted set data
        if (empty($this->data)){
            if(count($photoData['Tag'])){
                $tags='';
                foreach ($photoData['Tag'] as $tag){
                    $tags .= $tag['tag'].",";
                }
                $photoData['Photo']['Tags']=substr($tags,0,-1);
            }
            $this->data = $photoData;
        }

        // The form was posted handle the data
        if (!empty($this->params['data']['Photo'])){
            // Has a new picture been uploaded
            if($this->params['form']['photoFile']['error'] == 0){
                // Delete the old picture first
                $this->Photo->delPhoto($photoData['Photo']['filename']);
                // Try to save the new picture if successfull set the new file name
                $this->params['form']['photoFile']['name'] = time()."_".$this->params['form']['photoFile']['name'];
                $thumbSize = $this->Session->read('cheeseCake.thumbSize');
                $thumbType = $this->Session->read('cheeseCake.thumbType');
                if($this->Photo->savePhoto($this->params['form']['photoFile'],$thumbSize,$thumbType)){
                    $this->params['data']['Photo']['filename'] = $this->params['form']['photoFile']['name'];
                    $exif = serialize(read_exif_data_raw(WWW_ROOT."img/photos/".$this->params['form']['photoFile']['name'],"0"));
                    $this->params['data']['Photo']['exif'] = addslashes($exif);

                }
            }
            if (!empty($this->params['data']['Photo']['Tags'])){
                $this->params['data']['Tag']['Tag'] = $this->Photo->Tag->parseTags($this->params['data']['Photo']['Tags']);
            }
            // Try to save the data
            if($this->Photo->save($this->params['data'])){
                $this->Session->setFlash('Your Photo was updated successfully');
                $this->redirect("/view/".$id);
            }
        }
    }

    function delete($id)
    {
        $this->Photo->id= $id;
        $photoData = $this->Photo->read('filename');
        if($this->Photo->delPhoto($photoData['Photo']['filename'])){
            $this->Photo->del($id);
            $this->Session->setFlash('The selected Photo was deleted successfully');
            $this->redirect("/");
        }
    }


    function rss()
    {
        $this->Photo->unbindModel(array('hasMany'=>array('Comment')));
        $photoData = $this->Photo->findAll("ORDER BY id DESC",array('id','title','body','filename','created'),"",10);

        foreach($photoData as $photo){

            $link = $this->siteUrl."view/".$photo['Photo']['id'];
            $description = "<img src='".$this->siteUrl."img/thumbnails/".$photo['Photo']['filename']."' align=left />".$photo['Photo']['body'];
            $tags = array();
            foreach($photo['Tag'] as $tag){
                $tags[]= $tag['tag'];
            }
            $data[]=array('title'=> $photo['Photo']['title'],
                          'category'=>$tags,
                          'description'=>$description,
                          'link'=> $link,
                          'date'=>$photo['Photo']['created']);
        }
        $this->rss->feed($data,'Latest from '.$this->siteName,'', $this->siteUrl);
    }
}
?>